@@ -52,8 +52,8 @@ use std::cmp::{
 	PartialOrd,
 };
 use std::convert::{
-	AsRef,
 	AsMut,
+	AsRef,
 	From,
 };
 use std::fmt::{
@@ -62,6 +62,10 @@ use std::fmt::{
 	Display,
 	Formatter,
 };
+use std::hash::{
+	Hash,
+	Hasher,
+};
 use std::iter::{
 	DoubleEndedIterator,
 	ExactSizeIterator,
@@ -71,10 +75,12 @@ use std::iter::{
 use std::marker::PhantomData;
 use std::mem;
 use std::ops::{
+	AddAssign,
 	BitAndAssign,
 	BitOrAssign,
 	BitXorAssign,
 	Index,
+	Neg,
 	Not,
 	ShlAssign,
 	ShrAssign,
@@ -110,14 +116,15 @@ created by using the `From` implementation on `&BitSlice` and `&mut BitSlice`.
   downstream implementation, and can only be implemented in this crate.
 **/
 #[cfg_attr(nightly, repr(transparent))]
-pub struct BitSlice<E: Endian = BigEndian, T: Bits = u8> {
+pub struct BitSlice<E = BigEndian, T = u8>
+where E: Endian, T: Bits {
 	_endian: PhantomData<E>,
 	inner: [T],
 }
 
 impl<E, T> BitSlice<E, T>
 where E: Endian, T: Bits {
-	/// Gets the bit value at the given position.
+	/// Get the bit value at the given position.
 	///
 	/// The index value is a semantic count, not a bit address. It converts to a
 	/// bit position internally to this function.
@@ -136,7 +143,7 @@ where E: Endian, T: Bits {
 		self.as_ref()[elt].get(E::curr::<T>(bit))
 	}
 
-	/// Sets the bit value at the given position.
+	/// Set the bit value at the given position.
 	///
 	/// The index value is a semantic count, not a bit address. It converts to a
 	/// bit position internally to this function.
@@ -156,7 +163,211 @@ where E: Endian, T: Bits {
 		self.as_mut()[elt].set(E::curr::<T>(bit), value);
 	}
 
-	/// Returns the number of bits contained in the `BitSlice`.
+	/// Return true if *all* bits in the slice are set (logical `∧`).
+	///
+	/// # Truth Table
+	///
+	/// ```text
+	/// 0 0 => 0
+	/// 0 1 => 0
+	/// 1 0 => 0
+	/// 1 1 => 1
+	/// ```
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let all = bitvec![1; 10];
+	/// let any = bitvec![0, 0, 1, 0, 0];
+	/// let some = bitvec![1, 1, 0, 1, 1];
+	/// let none = bitvec![0; 10];
+	///
+	/// assert!(all.all());
+	/// assert!(!any.all());
+	/// assert!(!some.all());
+	/// assert!(!none.all());
+	/// ```
+	pub fn all(&self) -> bool {
+		//  Gallop the filled elements
+		let store = self.as_ref();
+		for elt in &store[.. self.elts()] {
+			if *elt != T::from(!0) {
+				return false;
+			}
+		}
+		//  Walk the partial tail
+		let bits = self.bits();
+		if bits > 0 {
+			let tail = store[self.elts()];
+			for bit in 0 .. bits {
+				if !tail.get(E::curr::<T>(bit)) {
+					return false;
+				}
+			}
+		}
+		return true;
+	}
+
+	/// Return true if *any* bit in the slice is set (logical `∨`).
+	///
+	/// # Truth Table
+	///
+	/// ```text
+	/// 0 0 => 0
+	/// 0 1 => 1
+	/// 1 0 => 1
+	/// 1 1 => 1
+	/// ```
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let all = bitvec![1; 10];
+	/// let any = bitvec![0, 0, 1, 0, 0];
+	/// let some = bitvec![1, 1, 0, 1, 1];
+	/// let none = bitvec![0; 10];
+	///
+	/// assert!(all.any());
+	/// assert!(any.any());
+	/// assert!(some.any());
+	/// assert!(!none.any());
+	/// ```
+	pub fn any(&self) -> bool {
+		//  Gallop the filled elements
+		let store = self.as_ref();
+		for elt in &store[.. self.elts()] {
+			if *elt != T::from(0) {
+				return true;
+			}
+		}
+		//  Walk the partial tail
+		let bits = self.bits();
+		if bits > 0 {
+			let tail = store[self.elts()];
+			for bit in 0 .. bits {
+				if tail.get(E::curr::<T>(bit)) {
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+
+	/// Return true if *any* bit in the slice is unset (logical `¬∧`).
+	///
+	/// # Truth Table
+	///
+	/// ```text
+	/// 0 0 => 1
+	/// 0 1 => 1
+	/// 1 0 => 1
+	/// 1 1 => 0
+	/// ```
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let all = bitvec![1; 10];
+	/// let any = bitvec![0, 0, 1, 0, 0];
+	/// let some = bitvec![1, 1, 0, 1, 1];
+	/// let none = bitvec![0; 10];
+	///
+	/// assert!(!all.not_all());
+	/// assert!(any.not_all());
+	/// assert!(some.not_all());
+	/// assert!(none.not_all());
+	/// ```
+	pub fn not_all(&self) -> bool {
+		!self.all()
+	}
+
+	/// Return true if *all* bits in the slice are uset (logical `¬∨`).
+	///
+	/// # Truth Table
+	///
+	/// ```text
+	/// 0 0 => 1
+	/// 0 1 => 0
+	/// 1 0 => 0
+	/// 1 1 => 0
+	/// ```
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let all = bitvec![1; 10];
+	/// let any = bitvec![0, 0, 1, 0, 0];
+	/// let some = bitvec![1, 1, 0, 1, 1];
+	/// let none = bitvec![0; 10];
+	///
+	/// assert!(!all.not_any());
+	/// assert!(!any.not_any());
+	/// assert!(!some.not_any());
+	/// assert!(none.not_any());
+	/// ```
+	pub fn not_any(&self) -> bool {
+		!self.any()
+	}
+
+	/// Return true if some, but not all, bits are set and some, but not all,
+	/// are unset.
+	///
+	/// # Truth Table
+	///
+	/// ```text
+	/// 0 0 => 0
+	/// 0 1 => 1
+	/// 1 0 => 1
+	/// 1 1 => 0
+	/// ```
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let all = bitvec![1; 2];
+	/// let some = bitvec![1, 0];
+	/// let none = bitvec![0; 2];
+	///
+	/// assert!(!all.some());
+	/// assert!(some.some());
+	/// assert!(!none.some());
+	/// ```
+	pub fn some(&self) -> bool {
+		self.any() && self.not_all()
+	}
+
+	/// Count how many bits are set high.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![1, 0, 1, 0, 1];
+	/// assert_eq!(bv.count_one(), 3);
+	/// ```
+	pub fn count_one(&self) -> usize {
+		self.into_iter().filter(|b| *b).count()
+	}
+
+	/// Count how many bits are set low.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![0, 1, 0, 1, 0];
+	/// assert_eq!(bv.count_zero(), 3);
+	/// ```
+	pub fn count_zero(&self) -> usize {
+		self.into_iter().filter(|b| !b).count()
+	}
+
+	/// Return the number of bits contained in the `BitSlice`.
 	///
 	/// # Examples
 	///
@@ -170,7 +381,7 @@ where E: Endian, T: Bits {
 		self.inner.len()
 	}
 
-	/// Counts how many *whole* storage elements are in the `BitSlice`.
+	/// Count how many *whole* storage elements are in the `BitSlice`.
 	///
 	/// If the `BitSlice` length is not an even multiple of the width of `T`,
 	/// then the slice under this `BitSlice` is one element longer than this
@@ -195,7 +406,7 @@ where E: Endian, T: Bits {
 		self.len() >> T::BITS
 	}
 
-	/// Counts how many bits are in the trailing partial storage element.
+	/// Count how many bits are in the trailing partial storage element.
 	///
 	/// If the `BitSlice` length is an even multiple of the width of `T`, then
 	/// this returns 0 and the `BitSlice` does not consider its final element to
@@ -220,7 +431,7 @@ where E: Endian, T: Bits {
 		self.len() as u8 & T::MASK
 	}
 
-	/// Returns `true` if the slice contains no bits.
+	/// Return `true` if the slice contains no bits.
 	///
 	/// # Examples
 	///
@@ -359,6 +570,113 @@ where E: Endian, T: Bits {
 	}
 }
 
+/// Clone a borrowed `BitSlice` into an owned `BitVec`.
+impl<E, T> ToOwned for BitSlice<E, T>
+where E: Endian, T: Bits {
+	type Owned = BitVec<E, T>;
+
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let src = bitvec![0; 5];
+	/// let src_ref: &BitSlice = &src;
+	/// let dst = src_ref.to_owned();
+	/// assert_eq!(src, dst);
+	/// ```
+	fn to_owned(&self) -> Self::Owned {
+		let mut out = Self::Owned::with_capacity(self.len());
+		unsafe {
+			let src = self.as_ptr();
+			let dst = out.as_mut_ptr();
+			let len = self.raw_len();
+			ptr::copy_nonoverlapping(src, dst, len);
+			out.set_len(self.len());
+		}
+		out
+	}
+}
+
+impl<E, T> Eq for BitSlice<E, T>
+where E: Endian, T: Bits {}
+
+impl<E, T> Ord for BitSlice<E, T>
+where E: Endian, T: Bits {
+	fn cmp(&self, rhs: &Self) -> Ordering {
+		match self.partial_cmp(rhs) {
+			Some(ord) => ord,
+			None => unreachable!("`BitSlice` has a total ordering"),
+		}
+	}
+}
+
+/// Test if two `BitSlice`s are semantically — not bitwise — equal.
+///
+/// It is valid to compare two slices of different endianness or element types.
+///
+/// The equality condition requires that they have the same number of total bits
+/// and that each pair of bits in semantic order are identical.
+impl<A, B, C, D> PartialEq<BitSlice<C, D>> for BitSlice<A, B>
+where A: Endian, B: Bits, C: Endian, D: Bits {
+	/// Perform a comparison by `==`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let l: BitVec<LittleEndian, u16> = bitvec![LittleEndian, u16; 0, 1, 0, 1];
+	/// let r: BitVec<BigEndian, u32> = bitvec![BigEndian, u32; 0, 1, 0, 1];
+	///
+	/// let ls: &BitSlice<_, _> = &l;
+	/// let rs: &BitSlice<_, _> = &r;
+	/// assert!(ls == rs);
+	/// ```
+	fn eq(&self, rhs: &BitSlice<C, D>) -> bool {
+		let (l, r) = (self.iter(), rhs.iter());
+		if l.len() != r.len() {
+			return false;
+		}
+		l.zip(r).all(|(l, r)| l == r)
+	}
+}
+
+/// Compare two `BitSlice`s by semantic — not bitwise — ordering.
+///
+/// The comparison sorts by testing each index for one slice to have a set bit
+/// where the other has an unset bit. If the slices are different, the slice
+/// with the set bit sorts greater than the slice with the unset bit.
+///
+/// If one of the slices is exhausted before they differ, the longer slice is
+/// greater.
+impl<A, B, C, D> PartialOrd<BitSlice<C, D>> for BitSlice<A, B>
+where A: Endian, B: Bits, C: Endian, D: Bits {
+	/// Perform a comparison by `<` or `>`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![0, 1, 0, 0];
+	/// let b = bitvec![0, 1, 0, 1];
+	/// let c = bitvec![0, 1, 0, 1, 1];
+	/// let aref: &BitSlice = &a;
+	/// let bref: &BitSlice = &b;
+	/// let cref: &BitSlice = &c;
+	/// assert!(aref < bref);
+	/// assert!(bref < cref);
+	/// ```
+	fn partial_cmp(&self, rhs: &BitSlice<C, D>) -> Option<Ordering> {
+		for (l, r) in self.iter().zip(rhs.iter()) {
+			match (l, r) {
+				(true, false) => return Some(Ordering::Greater),
+				(false, true) => return Some(Ordering::Less),
+				_ => continue,
+			}
+		}
+		self.len().partial_cmp(&rhs.len())
+	}
+}
+
 /// Give write access to all elements in the underlying storage, including the
 /// partially-filled tail element (if present).
 impl<E, T> AsMut<[T]> for BitSlice<E, T>
@@ -401,145 +719,6 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Performs the Boolean AND operation against another bitstream and writes the
-/// result into `self`. If the other bitstream ends before `self` does, it is
-/// extended with zero, clearing all remaining bits in `self`.
-impl<E, T, I> BitAndAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// AND a bitstream inta a slice.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
-	/// let rhs                =      bitvec![0, 0, 1, 1];
-	/// *lhs &= rhs;
-	/// assert_eq!("000100", &format!("{}", lhs));
-	/// ```
-	fn bitand_assign(&mut self, rhs: I) {
-		use std::iter::repeat;
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
-			let val = self.get(idx) & other;
-			self.set(idx, val);
-		}
-	}
-}
-
-/// Performs the Boolear OR operation against another bitstream and writes the
-/// result into `self`. If the other bitstream ends before `self` does, it is
-/// extended with zero, leaving all remaining bits in `self` as they were.
-impl<E, T, I> BitOrAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// OR a bitstream into a slice.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
-	/// let rhs                =      bitvec![0, 0, 1, 1];
-	/// *lhs |= rhs;
-	/// assert_eq!("011101", &format!("{}", lhs));
-	/// ```
-	fn bitor_assign(&mut self, rhs: I) {
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
-			let val = self.get(idx) | other;
-			self.set(idx, val);
-		}
-	}
-}
-
-/// Perform the Boolean XOR operation against another bitstream and writes the
-/// result into `self`. If the other bitstream ends before `self` does, it is
-/// extended with zero, leaving all remaining bits in `self` as they were.
-impl<E, T, I> BitXorAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// XOR a bitstream into a slice.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
-	/// let rhs                =      bitvec![0, 0, 1, 1];
-	/// *lhs ^= rhs;
-	/// assert_eq!("011001", &format!("{}", lhs));
-	/// ```
-	fn bitxor_assign(&mut self, rhs: I) {
-		use std::iter::repeat;
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
-			let val = self.get(idx) ^ other;
-			self.set(idx, val);
-		}
-	}
-}
-
-/// Print the `BitSlice` for debugging.
-///
-/// The output is of the form `BitSlice<E, T> [ELT, *]` where `<E, T>` is the
-/// endianness and element type, with square brackets on each end of the bits
-/// and all the elements of the array printed in binary. The printout is always
-/// in semantic order, and may not reflect the underlying buffer. To see the
-/// underlying buffer, use `.as_ref()`.
-///
-/// The alternate character `{:#?}` prints each element on its own line, rather
-/// than having all elements on the same line.
-impl<E, T> Debug for BitSlice<E, T>
-where E: Endian, T: Bits {
-	/// Render the `BitSlice` type header and contents for debug.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let bits: &BitSlice<LittleEndian, u16> = &bitvec![
-	///   LittleEndian, u16;
-	///   0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1,
-	///   0, 1
-	/// ];
-	/// assert_eq!(
-    ///     "BitSlice<LittleEndian, u16> [0101000011110101, 01]",
-	///     &format!("{:?}", bits)
-	/// );
-	/// ```
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		let alt = fmt.alternate();
-		self.fmt_header(fmt)?;
-		fmt.write_str(" [")?;
-		if alt { writeln!(fmt)?; }
-		self.fmt_body(fmt, true)?;
-		if alt { writeln!(fmt)?; }
-		fmt.write_str("]")
-	}
-}
-
-/// Prints the `BitSlice` for displaying.
-///
-/// This prints each element in turn, formatted in binary in semantic order (so
-/// the first bit seen is printed first and the last bit seen is printed last).
-/// Each element of storage is separated by a space for ease of reading.
-///
-/// The alternate character `{:#}` prints each element on its own line.
-///
-/// To see the in-memory representation, use `.as_ref()` to get access to the
-/// raw elements and print that slice instead.
-impl<E, T> Display for BitSlice<E, T>
-where E: Endian, T: Bits {
-	/// Renders the `BitSlice` contents for display.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let bits: &BitSlice = &bitvec![0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
-	/// assert_eq!("01001011 01", &format!("{}", bits));
-	/// ```
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		self.fmt_body(fmt, false)
-	}
-}
-
 /// Build a `BitSlice` from a slice of elements. The resulting `BitSlice` will
 /// always completely fill the original slice, and will not have a partial tail.
 impl<'a, E, T> From<&'a [T]> for &'a BitSlice<E, T>
@@ -605,6 +784,240 @@ where E: Endian, T: 'a + Bits {
 	}
 }
 
+/// Print the `BitSlice` for debugging.
+///
+/// The output is of the form `BitSlice<E, T> [ELT, *]` where `<E, T>` is the
+/// endianness and element type, with square brackets on each end of the bits
+/// and all the elements of the array printed in binary. The printout is always
+/// in semantic order, and may not reflect the underlying buffer. To see the
+/// underlying buffer, use `.as_ref()`.
+///
+/// The alternate character `{:#?}` prints each element on its own line, rather
+/// than having all elements on the same line.
+impl<E, T> Debug for BitSlice<E, T>
+where E: Endian, T: Bits {
+	/// Render the `BitSlice` type header and contents for debug.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bits: &BitSlice<LittleEndian, u16> = &bitvec![
+	///   LittleEndian, u16;
+	///   0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1,
+	///   0, 1
+	/// ];
+	/// assert_eq!(
+    ///     "BitSlice<LittleEndian, u16> [0101000011110101, 01]",
+	///     &format!("{:?}", bits)
+	/// );
+	/// ```
+	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
+		let alt = fmt.alternate();
+		self.fmt_header(fmt)?;
+		fmt.write_str(" [")?;
+		if alt { writeln!(fmt)?; fmt.write_str("    ")?; }
+		self.fmt_body(fmt, true)?;
+		if alt { writeln!(fmt)?; }
+		fmt.write_str("]")
+	}
+}
+
+/// Prints the `BitSlice` for displaying.
+///
+/// This prints each element in turn, formatted in binary in semantic order (so
+/// the first bit seen is printed first and the last bit seen is printed last).
+/// Each element of storage is separated by a space for ease of reading.
+///
+/// The alternate character `{:#}` prints each element on its own line.
+///
+/// To see the in-memory representation, use `.as_ref()` to get access to the
+/// raw elements and print that slice instead.
+impl<E, T> Display for BitSlice<E, T>
+where E: Endian, T: Bits {
+	/// Renders the `BitSlice` contents for display.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bits: &BitSlice = &bitvec![0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
+	/// assert_eq!("01001011 01", &format!("{}", bits));
+	/// ```
+	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
+		self.fmt_body(fmt, false)
+	}
+}
+
+impl<E, T> Hash for BitSlice<E, T>
+where E: Endian, T: Bits {
+	fn hash<H>(&self, hasher: &mut H)
+	where H: Hasher {
+		for bit in self {
+			hasher.write_u8(bit as u8);
+		}
+	}
+}
+
+/// Produce a read-only iterator over all the bits in the `BitSlice`.
+///
+/// This iterator follows the ordering in the `BitSlice` type, and implements
+/// `ExactSizeIterator` as `BitSlice` has a known, fixed length, and
+/// `DoubleEndedIterator` as it has known ends.
+impl<'a, E, T> IntoIterator for &'a BitSlice<E, T>
+where E: Endian, T: 'a + Bits {
+	type Item = bool;
+	type IntoIter = Iter<'a, E, T>;
+
+	/// Iterate over the slice.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![1, 0, 1, 0, 1, 1, 0, 0];
+	/// let bref: &BitSlice = &bv;
+	/// let mut count = 0;
+	/// for bit in bref {
+	///     if bit { count += 1; }
+	/// }
+	/// assert_eq!(count, 4);
+	/// ```
+	fn into_iter(self) -> Self::IntoIter {
+		self.into()
+	}
+}
+
+/// Perform unsigned addition in place on a `BitSlice`.
+///
+/// If the addend `BitSliec` is shorter than `self`, the addend is zero-extended
+/// to the right. If the addend is longer, the excess front length is unused.
+///
+/// Addition proceeds from the right ends of each slice towards the left.
+/// Because this trait is forbidden from returning anything, the final carry-out
+/// bit is discarded.
+///
+/// Note that, unlike `BitVec`, there is no subtraction implementation until I
+/// find a subtraction algorithm that does not require modifying the subtrahend.
+///
+/// Subtraction can be implemented by negating the intended subtrahend yourself,
+/// then using addition, or by using `BitVec`s instead of `BitSlice`s.
+impl<'a, E, T> AddAssign<&'a BitSlice<E, T>> for BitSlice<E, T>
+where E: Endian, T: Bits {
+	/// Perform unsigned wrapping addition in place.
+	///
+	/// # Examples
+	///
+	/// This example shows addition of a slice wrapping from MAX to zero.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let nums: [BitVec; 3] = [
+	///     bitvec![1, 1, 1, 0],
+	///     bitvec![1, 1, 1, 1],
+	///     bitvec![0, 0, 0, 0],
+	/// ];
+	/// let one = bitvec![0, 1];
+	/// let mut num = nums[0].clone();
+	/// let numr: &mut BitSlice = &mut num;
+	/// *numr += &one;
+	/// assert_eq!(numr, &nums[1] as &BitSlice);
+	/// *numr += &one;
+	/// assert_eq!(numr, &nums[2] as &BitSlice);
+	/// ```
+	fn add_assign(&mut self, addend: &'a BitSlice<E, T>) {
+		use std::iter::repeat;
+		//  zero-extend the addend if it's shorter than self
+		let mut addend_iter = addend.into_iter().rev().chain(repeat(false));
+		let mut c = false;
+		for place in (0 .. self.len()).rev() {
+			//  See BitVec::AddAssign
+			static JUMP: [u8; 8] = [0, 2, 2, 1, 2, 1, 1, 3];
+			let a = self.get(place);
+			let b = addend_iter.next().unwrap(); // addend is an infinite source
+			let idx = ((c as u8) << 2) | ((a as u8) << 1) | (b as u8);
+			let yz = JUMP[idx as usize];
+			let (y, z) = (yz & 2 != 0, yz & 1 != 0);
+			self.set(place, y);
+			c = z;
+		}
+	}
+}
+
+/// Perform the Boolean AND operation against another bitstream and writes the
+/// result into `self`. If the other bitstream ends before `self` does, it is
+/// extended with zero, clearing all remaining bits in `self`.
+impl<E, T, I> BitAndAssign<I> for BitSlice<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// AND a bitstream inta a slice.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
+	/// let rhs                =      bitvec![0, 0, 1, 1];
+	/// *lhs &= rhs;
+	/// assert_eq!("000100", &format!("{}", lhs));
+	/// ```
+	fn bitand_assign(&mut self, rhs: I) {
+		use std::iter::repeat;
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
+			let val = self.get(idx) & other;
+			self.set(idx, val);
+		}
+	}
+}
+
+/// Perform the Boolean OR operation against another bitstream and writes the
+/// result into `self`. If the other bitstream ends before `self` does, it is
+/// extended with zero, leaving all remaining bits in `self` as they were.
+impl<E, T, I> BitOrAssign<I> for BitSlice<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// OR a bitstream into a slice.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
+	/// let rhs                =      bitvec![0, 0, 1, 1];
+	/// *lhs |= rhs;
+	/// assert_eq!("011101", &format!("{}", lhs));
+	/// ```
+	fn bitor_assign(&mut self, rhs: I) {
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
+			let val = self.get(idx) | other;
+			self.set(idx, val);
+		}
+	}
+}
+
+/// Perform the Boolean XOR operation against another bitstream and writes the
+/// result into `self`. If the other bitstream ends before `self` does, it is
+/// extended with zero, leaving all remaining bits in `self` as they were.
+impl<E, T, I> BitXorAssign<I> for BitSlice<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// XOR a bitstream into a slice.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
+	/// let rhs                =      bitvec![0, 0, 1, 1];
+	/// *lhs ^= rhs;
+	/// assert_eq!("011001", &format!("{}", lhs));
+	/// ```
+	fn bitxor_assign(&mut self, rhs: I) {
+		use std::iter::repeat;
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
+			let val = self.get(idx) ^ other;
+			self.set(idx, val);
+		}
+	}
+}
+
 /// Index a single bit by semantic count. The index must be less than the length
 /// of the `BitSlice`.
 impl<'a, E, T> Index<usize> for &'a BitSlice<E, T>
@@ -660,32 +1073,86 @@ where E: Endian, T: 'a + Bits {
 	}
 }
 
-/// Produce a read-only iterator over all the bits in the `BitSlice`.
+/// Perform fixed-width 2's-complement negation of a `BitSlice`.
 ///
-/// This iterator follows the ordering in the `BitSlice` type, and implements
-/// `ExactSizeIterator` as `BitSlice` has a known, fixed length, and
-/// `DoubleEndedIterator` as it has known ends.
-impl<'a, E, T> IntoIterator for &'a BitSlice<E, T>
+/// Unlike the `!` operator (`Not` trait), the unary `-` operator treats the
+/// `BitSlice` as if it represents a signed 2's-complement integer of fixed
+/// width. The negation of a number in 2's complement is defined as its
+/// inversion (using `!`) plus one, and on fixed-width numbers has the following
+/// discontinuities:
+///
+/// - A slice whose bits are all zero is considered to represent the number zero
+///   which negates as itself.
+/// - A slice whose bits are all one is considered to represent the most
+///   negative number, which has no correpsonding positive number, and thus
+///   negates as zero.
+///
+/// This behavior was chosen so that all possible values would have *some*
+/// output, and so that repeated application converges at idempotence. The most
+/// negative input can never be reached by negation, but `--MOST_NEG` converges
+/// at the least unreasonable fallback value, 0.
+///
+/// Because `BitSlice` cannot move, the negation is performed in place.
+impl<'a, E, T> Neg for &'a mut BitSlice<E, T>
 where E: Endian, T: 'a + Bits {
-	type Item = bool;
-	type IntoIter = Iter<'a, E, T>;
+	type Output = Self;
 
-	/// Iterate over the slice.
+	/// Perform 2's-complement fixed-width negation.
 	///
 	/// # Examples
 	///
+	/// The contortions shown here are a result of this operator applying to a
+	/// mutable reference, and this example balancing access to the original
+	/// `BitVec` for comparison with aquiring a mutable borrow *as a slice* to
+	/// ensure that the `BitSlice` implementation is used, not the `BitVec`.
+	///
+	/// Negate an arbitrary positive number (first bit unset).
+	///
 	/// ```rust
 	/// use bitvec::*;
-	/// let bv = bitvec![1, 0, 1, 0, 1, 1, 0, 0];
-	/// let bref: &BitSlice = &bv;
-	/// let mut count = 0;
-	/// for bit in bref {
-	///     if bit { count += 1; }
-	/// }
-	/// assert_eq!(count, 4);
+	/// let mut num = bitvec![0, 1, 1, 0];
+	/// - (&mut num as &mut BitSlice);
+	/// assert_eq!(num, bitvec![1, 0, 1, 0]);
 	/// ```
-	fn into_iter(self) -> Self::IntoIter {
-		self.into()
+	///
+	/// Negate an arbitrary negative number. This example will use the above
+	/// result to demonstrate round-trip correctness.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut num = bitvec![1, 0, 1, 0];
+	/// - (&mut num as &mut BitSlice);
+	/// assert_eq!(num, bitvec![0, 1, 1, 0]);
+	/// ```
+	///
+	/// Negate the most negative number, which will become zero, and show
+	/// convergence at zero.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let zero = bitvec![0; 10];
+	/// let mut num = bitvec![1; 10];
+	/// - (&mut num as &mut BitSlice);
+	/// assert_eq!(num, zero);
+	/// - (&mut num as &mut BitSlice);
+	/// assert_eq!(num, zero);
+	/// ```
+	fn neg(self) -> Self::Output {
+		if self.is_empty() || self.not_any() {
+			return self;
+		}
+		Not::not(&mut *self);
+		//  Fill an element with all 1 bits
+		let elt: [T; 1] = [!T::default()];
+		if self.any() {
+			//  Turn a slice reference [T; 1] into a bit-slice reference [u1; 1]
+			let addend: &BitSlice<E, T> = {
+				unsafe { mem::transmute::<&[T], &BitSlice<E, T>>(&elt) }
+			};
+			//  And add it (if the slice was not all-ones).
+			AddAssign::add_assign(&mut *self, addend);
+		}
+		self
 	}
 }
 
@@ -722,86 +1189,6 @@ where E: Endian, T: 'a + Bits {
 	}
 }
 
-/// Test if two `BitSlice`s are semantically — not bitwise — equal.
-///
-/// It is valid to compare two slices of different endianness or element types.
-///
-/// The equality condition requires that they have the same number of total bits
-/// and that each pair of bits in semantic order are identical.
-impl<A, B, C, D> PartialEq<BitSlice<C, D>> for BitSlice<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
-	/// Perform a comparison by `==`.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let l: BitVec<LittleEndian, u16> = bitvec![LittleEndian, u16; 0, 1, 0, 1];
-	/// let r: BitVec<BigEndian, u32> = bitvec![BigEndian, u32; 0, 1, 0, 1];
-	///
-	/// let ls: &BitSlice<_, _> = &l;
-	/// let rs: &BitSlice<_, _> = &r;
-	/// assert!(ls == rs);
-	/// ```
-	fn eq(&self, rhs: &BitSlice<C, D>) -> bool {
-		let (l, r) = (self.iter(), rhs.iter());
-		if l.len() != r.len() {
-			return false;
-		}
-		l.zip(r).all(|(l, r)| l == r)
-	}
-}
-
-impl<E, T> Eq for BitSlice<E, T>
-where E: Endian, T: Bits {}
-
-/// Compare two `BitSlice`s by semantic — not bitwise — ordering.
-///
-/// The comparison sorts by testing each index for one slice to have a set bit
-/// where the other has an unset bit. If the slices are different, the slice
-/// with the set bit sorts greater than the slice with the unset bit.
-///
-/// If one of the slices is exhausted before they differ, the longer slice is
-/// greater.
-impl<A, B, C, D> PartialOrd<BitSlice<C, D>> for BitSlice<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
-	/// Perform a comparison by `<` or `>`.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let a = bitvec![0, 1, 0, 0];
-	/// let b = bitvec![0, 1, 0, 1];
-	/// let c = bitvec![0, 1, 0, 1, 1];
-	/// let aref: &BitSlice = &a;
-	/// let bref: &BitSlice = &b;
-	/// let cref: &BitSlice = &c;
-	/// assert!(aref < bref);
-	/// assert!(bref < cref);
-	/// ```
-	fn partial_cmp(&self, rhs: &BitSlice<C, D>) -> Option<Ordering> {
-		for (l, r) in self.iter().zip(rhs.iter()) {
-			match (l, r) {
-				(true, false) => return Some(Ordering::Greater),
-				(false, true) => return Some(Ordering::Less),
-				_ => continue,
-			}
-		}
-		self.len().partial_cmp(&rhs.len())
-	}
-}
-
-impl<E, T> Ord for BitSlice<E, T>
-where E: Endian, T: Bits {
-	fn cmp(&self, rhs: &Self) -> Ordering {
-		match self.partial_cmp(rhs) {
-			Some(ord) => ord,
-			None => unreachable!("`BitSlice` has a total ordering"),
-		}
-	}
-}
-
 __bitslice_shift!(u8, u16, u32, u64, i8, i16, i32, i64);
 
 /// Shift all bits in the array to the left — DOWN AND TOWARDS THE FRONT.
@@ -987,33 +1374,6 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Clone a borrowed `BitSlice` into an owned `BitVec`.
-impl<E, T> ToOwned for BitSlice<E, T>
-where E: Endian, T: Bits {
-	type Owned = BitVec<E, T>;
-
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let src = bitvec![0; 5];
-	/// let src_ref: &BitSlice = &src;
-	/// let dst = src_ref.to_owned();
-	/// assert_eq!(src, dst);
-	/// ```
-	fn to_owned(&self) -> Self::Owned {
-		let mut out = Self::Owned::with_capacity(self.len());
-		unsafe {
-			let src = self.as_ptr();
-			let dst = out.as_mut_ptr();
-			let len = self.raw_len();
-			ptr::copy_nonoverlapping(src, dst, len);
-			out.set_len(self.len());
-		}
-		out
-	}
-}
-
 /// Permit iteration over a `BitSlice`
 #[doc(hidden)]
 pub struct Iter<'a, E: 'a + Endian, T: 'a + Bits> {
