@@ -24,12 +24,17 @@ use std::convert::{
 	AsRef,
 	From,
 };
+use std::default::Default;
 use std::fmt::{
 	self,
 	Debug,
 	Display,
 	Formatter,
 };
+use std::hash::{
+	Hash,
+	Hasher,
+};
 use std::iter::{
 	DoubleEndedIterator,
 	ExactSizeIterator,
@@ -41,6 +46,8 @@ use std::iter::{
 use std::marker::PhantomData;
 use std::mem;
 use std::ops::{
+	Add,
+	AddAssign,
 	BitAnd,
 	BitAndAssign,
 	BitOr,
@@ -49,12 +56,16 @@ use std::ops::{
 	BitXorAssign,
 	Deref,
 	DerefMut,
+	Drop,
 	Index,
+	Neg,
 	Not,
 	Shl,
 	ShlAssign,
 	Shr,
 	ShrAssign,
+	Sub,
+	SubAssign,
 };
 use std::ptr;
 
@@ -260,7 +271,7 @@ where E: Endian, T: Bits {
 		self.do_with_vec(|v| v.shrink_to_fit());
 	}
 
-	/// Shrinks the `BitVec` to the given size, dropping all excess storage.
+	/// Shrink the `BitVec` to the given size, dropping all excess storage.
 	///
 	/// This does not affect the memory store! It will not zero the raw memory
 	/// nor will it deallocate.
@@ -434,201 +445,6 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Give write access to all live elements in the underlying storage, including
-/// the partially-filled tail.
-impl<E, T> AsMut<[T]> for BitVec<E, T>
-where E: Endian, T: Bits {
-	/// Access the underlying store.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let mut bv: BitVec = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
-	/// for elt in bv.as_mut() {
-	///   *elt += 2;
-	/// }
-	/// assert_eq!(&[2, 0b1000_0010], bv.as_ref());
-	/// ```
-	fn as_mut(&mut self) -> &mut [T] {
-		BitSlice::as_mut(self)
-	}
-}
-
-/// Give read access to all live elements in the underlying storage, including
-/// the partially-filled tail.
-impl<E, T> AsRef<[T]> for BitVec<E, T>
-where E: Endian, T: Bits {
-	/// Access the underlying store.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let bv = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
-	/// assert_eq!(&[0, 0b1000_0000], bv.as_ref());
-	/// ```
-	fn as_ref(&self) -> &[T] {
-		BitSlice::as_ref(self)
-	}
-}
-
-/// Perform the Boolean AND operation between each element of a `BitVec` and
-/// anything that can provide a stream of `bool` values (such as another
-/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
-/// have the length of the shorter sequence of bits -- if one is longer than the
-/// other, the extra bits will be ignored.
-impl<E, T, I> BitAnd<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	type Output = Self;
-
-	/// AND a vector and a bitstream, producing a new vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// let and = lhs & rhs;
-	/// assert_eq!("0001", &format!("{}", and));
-	/// ```
-	fn bitand(mut self, rhs: I) -> Self::Output {
-		self &= rhs;
-		self
-	}
-}
-
-/// Perform the Boolean AND operation in place on a `BitVec`, using a stream of
-/// `bool` values as the other bit for each operation. If the other stream is
-/// shorter than `self`, `self` will be truncated when the other stream expires.
-impl<E, T, I> BitAndAssign<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// AND another bitstream into a vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	///         src &= bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// assert_eq!("0001", &format!("{}", src));
-	/// ```
-	fn bitand_assign(&mut self, rhs: I) {
-		let mut len = 0;
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
-			let val = self.get(idx) & other;
-			self.set(idx, val);
-			len += 1;
-		}
-		self.truncate(len);
-	}
-}
-
-/// Perform the Boolean OR operation between each element of a `BitVec` and
-/// anything that can provide a stream of `bool` values (such as another
-/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
-/// have the length of the shorter sequence of bits -- if one is longer than the
-/// other, the extra bits will be ignored.
-impl<E, T, I> BitOr<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	type Output = Self;
-
-	/// OR a vector and a bitstream, producing a new vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// let or  = lhs | rhs;
-	/// assert_eq!("0111", &format!("{}", or));
-	/// ```
-	fn bitor(mut self, rhs: I) -> Self::Output {
-		self |= rhs;
-		self
-	}
-}
-
-/// Perform the Boolean OR operation in place on a `BitVec`, using a stream of
-/// `bool` values as the other bit for each operation. If the other stream is
-/// shorter than `self`, `self` will be truncated when the other stream expires.
-impl<E, T, I> BitOrAssign<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// OR another bitstream into a vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	///         src |= bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// assert_eq!("0111", &format!("{}", src));
-	/// ```
-	fn bitor_assign(&mut self, rhs: I) {
-		let mut len = 0;
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
-			let val = self.get(idx) | other;
-			self.set(idx, val);
-			len += 1;
-		}
-		self.truncate(len);
-	}
-}
-
-/// Perform the Boolean XOR operation between each element of a `BitVec` and
-/// anything that can provide a stream of `bool` values (such as another
-/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
-/// have the length of the shorter sequence of bits -- if one is longer than the
-/// other, the extra bits will be ignored.
-impl<E, T, I> BitXor<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	type Output = Self;
-
-	/// XOR a vector and a bitstream, producing a new vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// let xor = lhs ^ rhs;
-	/// assert_eq!("0110", &format!("{}", xor));
-	/// ```
-	fn bitxor(mut self, rhs: I) -> Self::Output {
-		self ^= rhs;
-		self
-	}
-}
-
-/// Perform the Boolean XOR operation in place on a `BitVec`, using a stream of
-/// `bool` values as the other bit for each operation. If the other stream is
-/// shorter than `self`, `self` will be truncated when the other stream expires.
-impl<E, T, I> BitXorAssign<I> for BitVec<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
-	/// XOR another bitstream into a vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
-	///         src ^= bitvec![BigEndian, u8; 0, 0, 1, 1];
-	/// assert_eq!("0110", &format!("{}", src));
-	/// ```
-	fn bitxor_assign(&mut self, rhs: I) {
-		let mut len = 0;
-		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
-			let val = self.get(idx) ^ other;
-			self.set(idx, val);
-			len += 1;
-		}
-		self.truncate(len);
-	}
-}
-
 /// Signify that `BitSlice` is the borrowed form of `BitVec`.
 impl<E, T> Borrow<BitSlice<E, T>> for BitVec<E, T>
 where E: Endian, T: Bits {
@@ -691,159 +507,103 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Print the `BitVec` for debugging.
-///
-/// The output is of the form `BitVec<E, T> [ELT, *]`, where `<E, T>` is the
-/// endianness and element type, with square brackets on each end of the bits
-/// and all the live elements in the vector printed in binary. The printout is
-/// always in semantic order, and may not reflect the underlying store. To see
-/// the underlying store, use `format!("{:?}", self.as_ref());` instead.
-///
-/// The alternate character `{:#?}` prints each element on its own line, rather
-/// than separated by a space.
-impl<E, T> Debug for BitVec<E, T>
+impl<E, T> Eq for BitVec<E, T>
+where E: Endian, T: Bits {}
+
+impl<E, T> Ord for BitVec<E, T>
 where E: Endian, T: Bits {
-	/// Render the `BitVec` type header and contents for debug.
+	fn cmp(&self, rhs: &Self) -> Ordering {
+		BitSlice::cmp(&self, &rhs)
+	}
+}
+
+/// Test if two `BitVec`s are semantically — not bitwise — equal.
+///
+/// It is valid to compare two vectors of different endianness or element types.
+///
+/// The equality condition requires that they have the same number of stored
+/// bits and that each pair of bits in semantic order are identical.
+impl<A, B, C, D> PartialEq<BitVec<C, D>> for BitVec<A, B>
+where A: Endian, B: Bits, C: Endian, D: Bits {
+	/// Perform a comparison by `==`.
 	///
 	/// # Examples
 	///
 	/// ```rust
 	/// use bitvec::*;
-	/// let bv = bitvec![LittleEndian, u16;
-	///     0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1
-	/// ];
-	/// assert_eq!(
-	///     "BitVec<LittleEndian, u16> [0101000011110101]",
-	///     &format!("{:?}", bv)
-	/// );
+	/// let l: BitVec<LittleEndian, u16> = bitvec![LittleEndian, u16; 0, 1, 0, 1];
+	/// let r: BitVec<BigEndian, u32> = bitvec![BigEndian, u32; 0, 1, 0, 1];
+	/// assert!(l == r);
 	/// ```
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		let alt = fmt.alternate();
-		self.fmt_header(fmt)?;
-		fmt.write_str(" [")?;
-		if alt { writeln!(fmt)?; }
-		self.fmt_body(fmt, true)?;
-		if alt { writeln!(fmt)?; }
-		fmt.write_str("]")
+	fn eq(&self, rhs: &BitVec<C, D>) -> bool {
+		BitSlice::eq(&self, &rhs)
 	}
 }
 
-/// Reborrow the `BitVec` as a `BitSlice`.
+/// Compare two `BitVec`s by semantic — not bitwise — ordering.
 ///
-/// This mimics the separation between `Vec<T>` and `[T]`.
-impl<E, T> Deref for BitVec<E, T>
-where E: Endian, T: Bits {
-	type Target = BitSlice<E, T>;
-
-	/// Dereference `&BitVec` down to `&BitSlice`.
+/// The comparison sorts by testing each index for one vector to have a set bit
+/// where the other vector has an unset bit. If the vectors are different, the
+/// vector with the set bit sorts greater than the vector with the unset bit.
+///
+/// If one of the vectors is exhausted before they differ, the longer vector is
+/// greater.
+impl<A, B, C, D> PartialOrd<BitVec<C, D>> for BitVec<A, B>
+where A: Endian, B: Bits, C: Endian, D: Bits {
+	/// Perform a comparison by `<` or `>`.
 	///
 	/// # Examples
 	///
 	/// ```rust
 	/// use bitvec::*;
-	/// let bv: BitVec = bitvec![1; 4];
-	/// let bref: &BitSlice = &bv;
-	/// assert!(bref.get(2));
+	/// use bitvec::*;
+	/// let a = bitvec![0, 1, 0, 0];
+	/// let b = bitvec![0, 1, 0, 1];
+	/// let c = bitvec![0, 1, 0, 1, 1];
+	/// assert!(a < b);
+	/// assert!(b < c);
 	/// ```
-	fn deref(&self) -> &Self::Target {
-		//  `BitVec`'s representation of its inner `Vec` matches exactly the
-		//  invariants of how `BitSlice` references must look. This is fine.
-		unsafe { mem::transmute(&self.inner as &[T]) }
+	fn partial_cmp(&self, rhs: &BitVec<C, D>) -> Option<Ordering> {
+		BitSlice::partial_cmp(&self, &rhs)
 	}
 }
 
-/// Reborrow the `BitVec` as a `BitSlice`.
-///
-/// This mimics the separation between `Vec<T>` and `[T]`.
-impl<E, T> DerefMut for BitVec<E, T>
+/// Give write access to all live elements in the underlying storage, including
+/// the partially-filled tail.
+impl<E, T> AsMut<[T]> for BitVec<E, T>
 where E: Endian, T: Bits {
-	/// Dereference `&mut BitVec` down to `&mut BitSlice`.
+	/// Access the underlying store.
 	///
 	/// # Examples
 	///
 	/// ```rust
 	/// use bitvec::*;
-	/// let mut bv: BitVec = bitvec![0; 6];
-	/// let bref: &mut BitSlice = &mut bv;
-	/// assert!(!bref.get(5));
-	/// bref.set(5, true);
-	/// assert!(bref.get(5));
+	/// let mut bv: BitVec = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
+	/// for elt in bv.as_mut() {
+	///   *elt += 2;
+	/// }
+	/// assert_eq!(&[2, 0b1000_0010], bv.as_ref());
 	/// ```
-	fn deref_mut(&mut self) -> &mut Self::Target {
-		unsafe { mem::transmute(&mut self.inner as &mut [T]) }
+	fn as_mut(&mut self) -> &mut [T] {
+		BitSlice::as_mut(self)
 	}
 }
 
-/// Print the `BitVec` for displaying.
-///
-/// This prints each element in turn, formatted in binary in semantic order (so
-/// the first bit seen is printed first and the last bit seen printed last).
-/// Each element of storage is separated by a space for ease of reading.
-///
-/// The alternate character `{:#}` prints each element on its own line.
-///
-/// To see the in-memory representation, use `AsRef` to get access to the raw
-/// elements and print that slice instead.
-impl<E, T> Display for BitVec<E, T>
+/// Give read access to all live elements in the underlying storage, including
+/// the partially-filled tail.
+impl<E, T> AsRef<[T]> for BitVec<E, T>
 where E: Endian, T: Bits {
-	/// Render the `BitVec` contents for display.
+	/// Access the underlying store.
 	///
 	/// # Examples
 	///
 	/// ```rust
 	/// use bitvec::*;
-	/// let bv = bitvec![BigEndian, u8; 0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
-	/// assert_eq!("01001011 01", &format!("{}", bv));
+	/// let bv = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
+	/// assert_eq!(&[0, 0b1000_0000], bv.as_ref());
 	/// ```
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		self.fmt_body(fmt, false)
-	}
-}
-
-/// Ready the underlying storage for Drop.
-impl<E, T> Drop for BitVec<E, T>
-where E: Endian, T: Bits {
-	fn drop(&mut self) {
-		//  If the `Vec` is non-empty, set the length to the number of used
-		//  elements as preparation for drop. The bits do not need to be wiped.
-		//
-		//  If we don't do this, the `Vec` drop will treat the bit total as the
-		//  number of elements and try to loop through all of them, which will
-		//  not take 2 ** T::BITS times as long to run as expected, because
-		//  it'll segfault.
-		let raw = self.raw_len();
-		unsafe { self.inner.set_len(raw); }
-	}
-}
-
-/// Extend a `BitVec` with the contents of another bitstream.
-///
-/// At present, this just calls `.push()` in a loop. When specialization becomes
-/// available, it will be able to more intelligently perform bulk moves from the
-/// source into `self` when the source is `BitSlice`-compatible.
-impl<E, T> Extend<bool> for BitVec<E, T>
-where E: Endian, T: Bits {
-	/// Extend a `BitVec` from another bitstream.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let mut bv = bitvec![0; 4];
-	/// bv.extend(bitvec![1; 4]);
-	/// assert_eq!("00001111", &format!("{}", bv));
-	/// ```
-	fn extend<I>(&mut self, src: I)
-	where I: IntoIterator<Item=bool> {
-		let iter = src.into_iter();
-		match iter.size_hint() {
-			(_, Some(hi)) => self.reserve(hi),
-			(lo, None) => self.reserve(lo),
-		}
-		for bit in iter {
-			self.push(bit);
-		}
-		self.shrink_to_fit();
+	fn as_ref(&self) -> &[T] {
+		BitSlice::as_ref(self)
 	}
 }
 
@@ -1003,6 +763,118 @@ impl<T: Bits> From<BitVec<BigEndian, T>> for BitVec<LittleEndian, T> {
 	}
 }
 
+impl<E, T> Default for BitVec<E, T>
+where E: Endian, T: Bits {
+	fn default() -> Self {
+		Self {
+			inner: Default::default(),
+			_endian: Default::default(),
+		}
+	}
+}
+
+/// Print the `BitVec` for debugging.
+///
+/// The output is of the form `BitVec<E, T> [ELT, *]`, where `<E, T>` is the
+/// endianness and element type, with square brackets on each end of the bits
+/// and all the live elements in the vector printed in binary. The printout is
+/// always in semantic order, and may not reflect the underlying store. To see
+/// the underlying store, use `format!("{:?}", self.as_ref());` instead.
+///
+/// The alternate character `{:#?}` prints each element on its own line, rather
+/// than separated by a space.
+impl<E, T> Debug for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Render the `BitVec` type header and contents for debug.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![LittleEndian, u16;
+	///     0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1
+	/// ];
+	/// assert_eq!(
+	///     "BitVec<LittleEndian, u16> [0101000011110101]",
+	///     &format!("{:?}", bv)
+	/// );
+	/// ```
+	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
+		let alt = fmt.alternate();
+		self.fmt_header(fmt)?;
+		fmt.write_str(" [")?;
+		if alt { writeln!(fmt)?; }
+		self.fmt_body(fmt, true)?;
+		if alt { writeln!(fmt)?; }
+		fmt.write_str("]")
+	}
+}
+
+/// Print the `BitVec` for displaying.
+///
+/// This prints each element in turn, formatted in binary in semantic order (so
+/// the first bit seen is printed first and the last bit seen printed last).
+/// Each element of storage is separated by a space for ease of reading.
+///
+/// The alternate character `{:#}` prints each element on its own line.
+///
+/// To see the in-memory representation, use `AsRef` to get access to the raw
+/// elements and print that slice instead.
+impl<E, T> Display for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Render the `BitVec` contents for display.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![BigEndian, u8; 0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
+	/// assert_eq!("01001011 01", &format!("{}", bv));
+	/// ```
+	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
+		self.fmt_body(fmt, false)
+	}
+}
+
+impl<E, T> Hash for BitVec<E, T>
+where E: Endian, T: Bits {
+	fn hash<H>(&self, hasher: &mut H)
+	where H: Hasher {
+		BitSlice::<E, T>::hash(&self, hasher)
+	}
+}
+
+/// Extend a `BitVec` with the contents of another bitstream.
+///
+/// At present, this just calls `.push()` in a loop. When specialization becomes
+/// available, it will be able to more intelligently perform bulk moves from the
+/// source into `self` when the source is `BitSlice`-compatible.
+impl<E, T> Extend<bool> for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Extend a `BitVec` from another bitstream.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut bv = bitvec![0; 4];
+	/// bv.extend(bitvec![1; 4]);
+	/// assert_eq!("00001111", &format!("{}", bv));
+	/// ```
+	fn extend<I>(&mut self, src: I)
+	where I: IntoIterator<Item=bool> {
+		let iter = src.into_iter();
+		match iter.size_hint() {
+			(_, Some(hi)) => self.reserve(hi),
+			(lo, None) => self.reserve(lo),
+		}
+		for bit in iter {
+			self.push(bit);
+		}
+		self.shrink_to_fit();
+	}
+}
+
 /// Permit the construction of a `BitVec` by using `.collect()` on an iterator
 /// of `bool`.
 impl<E, T> FromIterator<bool> for BitVec<E, T>
@@ -1031,6 +903,396 @@ where E: Endian, T: Bits {
 	}
 }
 
+/// Produce an iterator over all the bits in the vector.
+///
+/// This iterator follows the ordering in the vector type, and implements
+/// `ExactSizeIterator`, since `BitVec`s always know exactly how large they are,
+/// and `DoubleEndedIterator`, since they have known ends.
+impl<E, T> IntoIterator for BitVec<E, T>
+where E: Endian, T: Bits {
+	type Item = bool;
+	#[doc(hidden)]
+	type IntoIter = IntoIter<E, T>;
+
+	/// Iterate over the vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv = bitvec![BigEndian, u8; 1, 1, 1, 1, 0, 0, 0, 0];
+	/// let mut count = 0;
+	/// for bit in bv {
+	///     if bit { count += 1; }
+	/// }
+	/// assert_eq!(count, 4);
+	/// ```
+	fn into_iter(self) -> Self::IntoIter {
+		Self::IntoIter::from(self)
+	}
+}
+
+/// Add two `BitVec`s together, zero-extending the shorter.
+///
+/// `BitVec` addition works just like adding numbers longhand on paper. The
+/// first bits in the `BitVec` are the highest, so addition works from right to
+/// left, and the shorter `BitVec` is assumed to be extended to the left with
+/// zero.
+///
+/// The output `BitVec` may be one bit longer than the longer input, if addition
+/// overflowed.
+///
+/// Numeric arithmetic is provided on `BitVec` as a convenience. Serious numeric
+/// computation on variable-length integers should use the `num_bigint` crate
+/// instead, which is written specifically for that use case. `BitVec`s are not
+/// intended for arithmetic, and `bitvec` makes no guarantees about sustained
+/// correctness in arithmetic at this time.
+impl<E, T> Add for BitVec<E, T>
+where E: Endian, T: Bits {
+	type Output = Self;
+
+	/// Add two `BitVec`s.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![0, 1, 0, 1];
+	/// let b = bitvec![0, 0, 1, 1];
+	/// let s = a + b;
+	/// assert_eq!(bitvec![1, 0, 0, 0], s);
+	/// ```
+	///
+	/// This example demonstrates the addition of differently-sized `BitVec`s,
+	/// and will overflow.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![1; 4];
+	/// let b = bitvec![1; 1];
+	/// let s = b + a;
+	/// assert_eq!(bitvec![1, 0, 0, 0, 0], s);
+	/// ```
+	fn add(mut self, addend: Self) -> Self::Output {
+		self += addend;
+		self
+	}
+}
+
+/// Add another `BitVec` into `self`, zero-extending the shorter.
+///
+/// `BitVec` addition works just like adding numbers longhand on paper. The
+/// first bits in the `BitVec` are the highest, so addition works from right to
+/// left, and the shorter `BitVec` is assumed to be extended to the left with
+/// zero.
+///
+/// The output `BitVec` may be one bit longer than the longer input, if addition
+/// overflowed.
+///
+/// Numeric arithmetic is provided on `BitVec` as a convenience. Serious numeric
+/// computation on variable-length integers should use the `num_bigint` crate
+/// instead, which is written specifically for that use case. `BitVec`s are not
+/// intended for arithmetic, and `bitvec` makes no guarantees about sustained
+/// correctness in arithmetic at this time.
+impl<E, T> AddAssign for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Add another `BitVec` into `self`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut a = bitvec![1, 0, 0, 1];
+	/// let b = bitvec![0, 1, 1, 1];
+	/// a += b;
+	/// assert_eq!(a, bitvec![1, 0, 0, 0, 0]);
+	/// ```
+	fn add_assign(&mut self, mut addend: Self) {
+		use std::iter::repeat;
+		//  If the other vec is longer, swap them and try again.
+		if addend.len() > self.len() {
+			mem::swap(self, &mut addend);
+			return *self += addend;
+		}
+		//  Now that self.len() >= addend.len(), proceed with addition.
+		//
+		//  I don't, at this time, want to implement a carry-lookahead adder in
+		//  software, so this is going to be a plain ripple-carry adder with
+		//  O(n) runtime. Furthermore, until I think of an optimization
+		//  strategy, it is going to build up another bitvec to use as a stack.
+		//
+		//  Computers are fast. Whatever.
+		let mut c = false;
+		let mut stack = BitVec::<E, T>::with_capacity(self.len());
+		//  Reverse self, reverse addend and zero-extend, and zip both together.
+		//  This walks both vecs from rightmost to leftmost, and considers an
+		//  early expiration of addend to continue with 0 bits.
+		//
+		//  100111
+		// +  0010
+		//  ^^---- semantically zero
+		for (a, b) in self.iter().rev().zip(addend.into_iter().rev().chain(repeat(false))) {
+			//  Addition is a finite state machine that can be precomputed into a single
+			//  jump table rather than requiring more complex branching.
+			//  The table is indexed as (carry, a, b) and returns (bit, carry).
+			static JUMP: [u8; 8] = [
+				//  0 + 0 + 0 = 0, 0
+				0,
+				//  0 + 1 + 0 = 1, 0
+				2,
+				//  1 + 0 + 0 = 1, 0
+				2,
+				//  1 + 1 + 1 = 0, 1
+				1,
+				//  0 + 0 + 1 = 1, 0
+				2,
+				//  0 + 1 + 0 = 0, 1
+				1,
+				//  1 + 0 + 0 = 0, 1
+				1,
+				//  1 + 1 + 1 = 1, 1
+				3,
+			];
+			let idx = ((c as u8) << 2) | ((a as u8) << 1) | (b as u8);
+			let yz = JUMP[idx as usize];
+			let (y, z) = (yz & 2 != 0, yz & 1 != 0);
+			//  Note: I checked in Godbolt, and the above comes out to ten
+			//  simple instructions with the JUMP baked in as immediate values.
+			//  The more semantically clear match statement does not optimize
+			//  nearly as well.
+			stack.push(y);
+			c = z;
+		}
+		//  If the carry made it to the end, push it.
+		if c {
+			stack.push(true);
+		}
+		//  Unwind the stack into `self`.
+		self.clear();
+		while let Some(bit) = stack.pop() {
+			self.push(bit);
+		}
+	}
+}
+
+/// Perform the Boolean AND operation between each element of a `BitVec` and
+/// anything that can provide a stream of `bool` values (such as another
+/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
+/// have the length of the shorter sequence of bits -- if one is longer than the
+/// other, the extra bits will be ignored.
+impl<E, T, I> BitAnd<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	type Output = Self;
+
+	/// AND a vector and a bitstream, producing a new vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// let and = lhs & rhs;
+	/// assert_eq!("0001", &format!("{}", and));
+	/// ```
+	fn bitand(mut self, rhs: I) -> Self::Output {
+		self &= rhs;
+		self
+	}
+}
+
+/// Perform the Boolean AND operation in place on a `BitVec`, using a stream of
+/// `bool` values as the other bit for each operation. If the other stream is
+/// shorter than `self`, `self` will be truncated when the other stream expires.
+impl<E, T, I> BitAndAssign<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// AND another bitstream into a vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	///         src &= bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// assert_eq!("0001", &format!("{}", src));
+	/// ```
+	fn bitand_assign(&mut self, rhs: I) {
+		let mut len = 0;
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
+			let val = self.get(idx) & other;
+			self.set(idx, val);
+			len += 1;
+		}
+		self.truncate(len);
+	}
+}
+
+/// Perform the Boolean OR operation between each element of a `BitVec` and
+/// anything that can provide a stream of `bool` values (such as another
+/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
+/// have the length of the shorter sequence of bits -- if one is longer than the
+/// other, the extra bits will be ignored.
+impl<E, T, I> BitOr<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	type Output = Self;
+
+	/// OR a vector and a bitstream, producing a new vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// let or  = lhs | rhs;
+	/// assert_eq!("0111", &format!("{}", or));
+	/// ```
+	fn bitor(mut self, rhs: I) -> Self::Output {
+		self |= rhs;
+		self
+	}
+}
+
+/// Perform the Boolean OR operation in place on a `BitVec`, using a stream of
+/// `bool` values as the other bit for each operation. If the other stream is
+/// shorter than `self`, `self` will be truncated when the other stream expires.
+impl<E, T, I> BitOrAssign<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// OR another bitstream into a vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	///         src |= bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// assert_eq!("0111", &format!("{}", src));
+	/// ```
+	fn bitor_assign(&mut self, rhs: I) {
+		let mut len = 0;
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
+			let val = self.get(idx) | other;
+			self.set(idx, val);
+			len += 1;
+		}
+		self.truncate(len);
+	}
+}
+
+/// Perform the Boolean XOR operation between each element of a `BitVec` and
+/// anything that can provide a stream of `bool` values (such as another
+/// `BitVec`, or any `bool` generator of your choice). The `BitVec` emitted will
+/// have the length of the shorter sequence of bits -- if one is longer than the
+/// other, the extra bits will be ignored.
+impl<E, T, I> BitXor<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	type Output = Self;
+
+	/// XOR a vector and a bitstream, producing a new vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let lhs = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	/// let rhs = bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// let xor = lhs ^ rhs;
+	/// assert_eq!("0110", &format!("{}", xor));
+	/// ```
+	fn bitxor(mut self, rhs: I) -> Self::Output {
+		self ^= rhs;
+		self
+	}
+}
+
+/// Perform the Boolean XOR operation in place on a `BitVec`, using a stream of
+/// `bool` values as the other bit for each operation. If the other stream is
+/// shorter than `self`, `self` will be truncated when the other stream expires.
+impl<E, T, I> BitXorAssign<I> for BitVec<E, T>
+where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+	/// XOR another bitstream into a vector.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut src  = bitvec![BigEndian, u8; 0, 1, 0, 1];
+	///         src ^= bitvec![BigEndian, u8; 0, 0, 1, 1];
+	/// assert_eq!("0110", &format!("{}", src));
+	/// ```
+	fn bitxor_assign(&mut self, rhs: I) {
+		let mut len = 0;
+		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
+			let val = self.get(idx) ^ other;
+			self.set(idx, val);
+			len += 1;
+		}
+		self.truncate(len);
+	}
+}
+
+/// Reborrow the `BitVec` as a `BitSlice`.
+///
+/// This mimics the separation between `Vec<T>` and `[T]`.
+impl<E, T> Deref for BitVec<E, T>
+where E: Endian, T: Bits {
+	type Target = BitSlice<E, T>;
+
+	/// Dereference `&BitVec` down to `&BitSlice`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let bv: BitVec = bitvec![1; 4];
+	/// let bref: &BitSlice = &bv;
+	/// assert!(bref.get(2));
+	/// ```
+	fn deref(&self) -> &Self::Target {
+		//  `BitVec`'s representation of its inner `Vec` matches exactly the
+		//  invariants of how `BitSlice` references must look. This is fine.
+		unsafe { mem::transmute(&self.inner as &[T]) }
+	}
+}
+
+/// Reborrow the `BitVec` as a `BitSlice`.
+///
+/// This mimics the separation between `Vec<T>` and `[T]`.
+impl<E, T> DerefMut for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Dereference `&mut BitVec` down to `&mut BitSlice`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let mut bv: BitVec = bitvec![0; 6];
+	/// let bref: &mut BitSlice = &mut bv;
+	/// assert!(!bref.get(5));
+	/// bref.set(5, true);
+	/// assert!(bref.get(5));
+	/// ```
+	fn deref_mut(&mut self) -> &mut Self::Target {
+		unsafe { mem::transmute(&mut self.inner as &mut [T]) }
+	}
+}
+
+/// Ready the underlying storage for Drop.
+impl<E, T> Drop for BitVec<E, T>
+where E: Endian, T: Bits {
+	fn drop(&mut self) {
+		//  If the `Vec` is non-empty, set the length to the number of used
+		//  elements as preparation for drop. The bits do not need to be wiped.
+		//
+		//  If we don't do this, the `Vec` drop will treat the bit total as the
+		//  number of elements and try to loop through all of them, which will
+		//  not take 2 ** T::BITS times as long to run as expected, because
+		//  it'll segfault.
+		let raw = self.raw_len();
+		unsafe { self.inner.set_len(raw); }
+	}
+}
+
 /// Get the bit at a specific index. The index must be less than the length of
 /// the `BitVec`.
 impl<E, T> Index<usize> for BitVec<E, T>
@@ -1100,32 +1362,29 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Produce an iterator over all the bits in the vector.
+/// 2's-complement negation of a `BitVec`.
 ///
-/// This iterator follows the ordering in the vector type, and implements
-/// `ExactSizeIterator`, since `BitVec`s always know exactly how large they are,
-/// and `DoubleEndedIterator`, since they have known ends.
-impl<E, T> IntoIterator for BitVec<E, T>
+/// In 2's-complement, negation is defined as bit-inversion followed by adding
+/// one.
+///
+/// Numeric arithmetic is provided on `BitVec` as a convenience. Serious numeric
+/// computation on variable-length integers should use the `num_bigint` crate
+/// instead, which is written specifically for that use case. `BitVec`s are not
+/// intended for arithmetic, and `bitvec` makes no guarantees about sustained
+/// correctness in arithmetic at this time.
+impl<E, T> Neg for BitVec<E, T>
 where E: Endian, T: Bits {
-	type Item = bool;
-	#[doc(hidden)]
-	type IntoIter = IntoIter<E, T>;
+	type Output = Self;
 
-	/// Iterate over the vector.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let bv = bitvec![BigEndian, u8; 1, 1, 1, 1, 0, 0, 0, 0];
-	/// let mut count = 0;
-	/// for bit in bv {
-	///     if bit { count += 1; }
-	/// }
-	/// assert_eq!(count, 4);
-	/// ```
-	fn into_iter(self) -> Self::IntoIter {
-		Self::IntoIter::from(self)
+	fn neg(mut self) -> Self::Output {
+		//  An empty vector does nothing.
+		//  Negative zero is zero. Without this check, -[0+] becomes[10+1].
+		if self.is_empty() || self.not_any() {
+			return self;
+		}
+		self = !self;
+		self += BitVec::<E, T>::from(&[true] as &[bool]);
+		self
 	}
 }
 
@@ -1159,67 +1418,6 @@ where E: Endian, T: Bits {
 	}
 }
 
-/// Test if two `BitVec`s are semantically — not bitwise — equal.
-///
-/// It is valid to compare two vectors of different endianness or element types.
-///
-/// The equality condition requires that they have the same number of stored
-/// bits and that each pair of bits in semantic order are identical.
-impl<A, B, C, D> PartialEq<BitVec<C, D>> for BitVec<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
-	/// Perform a comparison by `==`.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// let l: BitVec<LittleEndian, u16> = bitvec![LittleEndian, u16; 0, 1, 0, 1];
-	/// let r: BitVec<BigEndian, u32> = bitvec![BigEndian, u32; 0, 1, 0, 1];
-	/// assert!(l == r);
-	/// ```
-	fn eq(&self, rhs: &BitVec<C, D>) -> bool {
-		BitSlice::eq(&self, &rhs)
-	}
-}
-
-impl<E, T> Eq for BitVec<E, T>
-where E: Endian, T: Bits {}
-
-/// Compare two `BitVec`s by semantic — not bitwise — ordering.
-///
-/// The comparison sorts by testing each index for one vector to have a set bit
-/// where the other vector has an unset bit. If the vectors are different, the
-/// vector with the set bit sorts greater than the vector with the unset bit.
-///
-/// If one of the vectors is exhausted before they differ, the longer vector is
-/// greater.
-impl<A, B, C, D> PartialOrd<BitVec<C, D>> for BitVec<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
-	/// Perform a comparison by `<` or `>`.
-	///
-	/// # Examples
-	///
-	/// ```rust
-	/// use bitvec::*;
-	/// use bitvec::*;
-	/// let a = bitvec![0, 1, 0, 0];
-	/// let b = bitvec![0, 1, 0, 1];
-	/// let c = bitvec![0, 1, 0, 1, 1];
-	/// assert!(a < b);
-	/// assert!(b < c);
-	/// ```
-	fn partial_cmp(&self, rhs: &BitVec<C, D>) -> Option<Ordering> {
-		BitSlice::partial_cmp(&self, &rhs)
-	}
-}
-
-impl<E, T> Ord for BitVec<E, T>
-where E: Endian, T: Bits {
-	fn cmp(&self, rhs: &Self) -> Ordering {
-		BitSlice::cmp(&self, &rhs)
-	}
-}
-
 __bitvec_shift!(u8, u16, u32, u64, i8, i16, i32, i64);
 
 /// Shift all bits in the vector to the left – DOWN AND TOWARDS THE FRONT.
@@ -1445,7 +1643,6 @@ where E: Endian, T: Bits {
 	/// ```
 	fn shr_assign(&mut self, shamt: usize) {
 		let old_len = self.len();
-		//  Implement `Extend` to make this more efficient
 		for _ in 0 .. shamt {
 			self.push(false);
 		}
@@ -1459,6 +1656,140 @@ where E: Endian, T: Bits {
 	}
 }
 
+/// Subtract one `BitVec` from another assuming 2's-complement encoding.
+///
+/// Subtraction is a more complex operation than addition. The bit-level work is
+/// largely the same, but semantic distinctions must be made. Unlike addition,
+/// which is commutative and tolerant of switching the order of the addends,
+/// subtraction cannot swap the minuend (LHS) and subtrahend (RHS).
+///
+/// Because of the properties of 2's-complement arithmetic, M - S is equivalent
+/// to M + (!S + 1). Subtraction therefore bitflips the subtrahend and adds one.
+/// This may, in a degenerate case, cause the subtrahend to increase in length.
+///
+/// Once the subtrahend is stable, the minuend zero-extends its left side in
+/// order to match the length of the subtrahend if needed (this is provided by
+/// the `>>` operator).
+///
+/// When the minuend is stable, the minuend and subtrahend are added together
+/// by the `<BitVec as Add>` implementation. The output will be encoded in
+/// 2's-complement, so a leading one means that the output is considered
+/// negative.
+///
+/// Interpreting the contents of a `BitVec` as an integer is beyond the scope of
+/// this crate.
+///
+/// Numeric arithmetic is provided on `BitVec` as a convenience. Serious numeric
+/// computation on variable-length integers should use the `num_bigint` crate
+/// instead, which is written specifically for that use case. `BitVec`s are not
+/// intended for arithmetic, and `bitvec` makes no guarantees about sustained
+/// correctness in arithmetic at this time.
+impl<E, T> Sub for BitVec<E, T>
+where E: Endian, T: Bits {
+	type Output = Self;
+
+	/// Subtract one `BitVec` from another.
+	///
+	/// # Examples
+	///
+	/// Minuend larger than subtrahend, positive difference.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![1, 0];
+	/// let b = bitvec![   1];
+	/// let c = a - b;
+	/// assert_eq!(bitvec![0, 1], c);
+	/// ```
+	///
+	/// Minuend smaller than subtrahend, negative difference.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![   1];
+	/// let b = bitvec![1, 0];
+	/// let c = a - b;
+	/// assert_eq!(bitvec![1, 1], c);
+	/// ```
+	///
+	/// Subtraction from self is correctly handled.
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![1; 4];
+	/// let b = a.clone();
+	/// let c = a - b;
+	/// assert!(c.not_any(), "{:?}", c);
+	/// ```
+	fn sub(mut self, subtrahend: Self) -> Self::Output {
+		self -= subtrahend;
+		self
+	}
+}
+
+/// Subtract another `BitVec` from `self`, assuming 2's-complement encoding.
+///
+/// The minuend is zero-extended, or the subtrahend sign-extended, as needed to
+/// ensure that the vectors are the same width before subtraction occurs.
+///
+/// The `Sub` trait has more documentation on the subtraction process.
+///
+/// Numeric arithmetic is provided on `BitVec` as a convenience. Serious numeric
+/// computation on variable-length integers should use the `num_bigint` crate
+/// instead, which is written specifically for that use case. `BitVec`s are not
+/// intended for arithmetic, and `bitvec` makes no guarantees about sustained
+/// correctness in arithmetic at this time.
+impl<E, T> SubAssign for BitVec<E, T>
+where E: Endian, T: Bits {
+	/// Subtract another `BitVec` from `self`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::*;
+	/// let a = bitvec![0, 0, 0, 1];
+	/// let b = bitvec![0, 0, 0, 0];
+	/// let c = a - b;
+	/// assert_eq!(c, bitvec![0, 0, 0, 1]);
+	/// ```
+	fn sub_assign(&mut self, mut subtrahend: Self) {
+		//  Test for a zero subtrahend. Subtraction of zero is the identity
+		//  function, and can exit immediately.
+		if subtrahend.not_any() {
+			return;
+		}
+		//  Invert the subtrahend in preparation for addition
+		subtrahend = -subtrahend;
+		let (llen, rlen) = (self.len(), subtrahend.len());
+		//  If the subtrahend is longer than the minuend, 0-extend the minuend.
+		if rlen > llen {
+			let diff = rlen - llen;
+			*self >>= diff;
+			*self += subtrahend;
+		}
+		else {
+			//  If the minuend is longer than the subtrahend, 1-extend the
+			//  subtrahend.
+			if llen > rlen {
+				let diff = llen - rlen;
+				let sign = subtrahend.get(0);
+				subtrahend >>= diff;
+				//  Implementing BitVec >> (usize, bool) would permit sign
+				//  extension in fewer steps.
+				for idx in 0 .. diff {
+					subtrahend.set(idx, sign);
+				}
+			}
+			let old = self.len();
+			*self += subtrahend;
+			//  If the subtraction emitted a carry, remove it.
+			if self.len() > old {
+				*self <<= 1;
+			}
+		}
+	}
+}
+
 /// Iterate over an owned `BitVec`.
 #[doc(hidden)]
 pub struct IntoIter<E, T>
